@@ -188,13 +188,15 @@ class QA(ABC):
 
 
 
-    def ask(self, question, batch_size=8, n_docs_considered=10, n_answers=50, 
+    def ask(self, question, query=None, batch_size=8, n_docs_considered=10, n_answers=50, 
             rerank_threshold=0.015, include_np=False):
         """
+        ```
         submit question to obtain candidate answers
 
         Args:
           question(str): question in the form of a string
+          query(str): Optional. If not None, words in query will be used to retrieve contexts instead of words in question
           batch_size(int):  number of question-context pairs fed to model at each iteration
                             Default:8
                             Increase for faster answer-retrieval.
@@ -217,14 +219,15 @@ class QA(ABC):
                              Default:False
         Returns:
           list
+        ```
         """
         # locate candidate document contexts
         paragraphs = []
         refs = []
         #doc_results = self.search(question, limit=n_docs_considered)
-        doc_results = self.search(_process_question(question, include_np=include_np), limit=n_docs_considered)
+        doc_results = self.search(_process_question(query if query is not None else question, include_np=include_np), limit=n_docs_considered)
         if not doc_results: 
-            warnings.warn('No documents matched words in question')
+            warnings.warn('No documents matched words in question (or query if supplied)')
             return []
         # extract paragraphs as contexts
         contexts = []
@@ -281,7 +284,7 @@ class QA(ABC):
 
                 mb.child.comment = f'generating candidate answers'
 
-
+        if not answers: return answers # fix for #307
         answers = sorted(answers, key = lambda k:k['confidence'], reverse=True)
         if n_answers is not None:
             answers = answers[:n_answers]
@@ -334,12 +337,13 @@ class SimpleQA(QA):
                  bert_squad_model='bert-large-uncased-whole-word-masking-finetuned-squad',
                  bert_emb_model='bert-base-uncased'):
         """
+        ```
         SimpleQA constructor
         Args:
           index_dir(str):  path to index directory created by SimpleQA.initialze_index
           bert_squad_model(str): name of BERT SQUAD model to use
           bert_emb_model(str): BERT model to use to generate embeddings for semantic similarity
-
+        ```
         """
 
         self.index_dir = index_dir
@@ -368,6 +372,7 @@ class SimpleQA(QA):
     def index_from_list(cls, docs, index_dir, commit_every=1024, breakup_docs=True,
                         procs=1, limitmb=256, multisegment=False, min_words=20, references=None):
         """
+        ```
         index documents from list.
         The procs, limitmb, and especially multisegment arguments can be used to 
         speed up indexing, if it is too slow.  Please see the whoosh documentation
@@ -395,6 +400,7 @@ class SimpleQA(QA):
                             displays candidate answers in a pandas DataFRame.
 
                             If references is None, the index of element in docs is used as reference.
+        ```
         """
         if not isinstance(docs, (np.ndarray, list)): raise ValueError('docs must be a list of strings')
         if references is not None and not isinstance(references, (np.ndarray, list)): raise ValueError('references must be a list of strings')
@@ -436,6 +442,7 @@ class SimpleQA(QA):
     def index_from_folder(cls, folder_path, index_dir,  use_text_extraction=False, commit_every=1024, breakup_docs=True, 
                           min_words=20, encoding='utf-8', procs=1, limitmb=256, multisegment=False, verbose=1):
         """
+        ```
         index all plain text documents within a folder.
         The procs, limitmb, and especially multisegment arguments can be used to 
         speed up indexing, if it is too slow.  Please see the whoosh documentation
@@ -458,7 +465,7 @@ class SimpleQA(QA):
           limitmb(int): memory limit in MB for each process
           multisegment(bool): new segments written instead of merging
           verbose(bool): verbosity
-
+        ```
         """
         if use_text_extraction:
             try:
@@ -512,12 +519,14 @@ class SimpleQA(QA):
 
     def search(self, query, limit=10):
         """
+        ```
         search index for query
         Args:
           query(str): search query
           limit(int):  number of top search results to return
         Returns:
           list of dicts with keys: reference, rawtext
+        ```
         """
         ix = self._open_ix()
         with ix.searcher() as searcher:
