@@ -66,6 +66,13 @@ def metrics_from_model(model):
             except:
                 warnings.warn(msg)
                 return []
+        elif isinstance(mlist, list) and hasattr(mlist[0], 'name'): # tf.keras.metrics.AUC()
+            try:
+                return [m.name for m in mlist]
+            except:
+                warnings.warn(msg)
+                return []
+
         else:
             warnings.warn(msg)
             return []
@@ -98,6 +105,8 @@ def is_classifier(model):
         mlist = metrics_from_model(model)
         if isinstance(mlist, (list, np.ndarray)) and any(['accuracy' in m for m in mlist]):
             is_classifier = True
+        elif isinstance(mlist, (list, np.ndarray)) and any(['auc' in m for m in mlist]):
+            is_classifier = True
 
     # check for multilabel
     if loss == 'binary_crossentropy':
@@ -535,6 +544,7 @@ def get_hf_model_name(model_id):
 class YTransform:
     def __init__(self, class_names=[], label_encoder=None):
         """
+        ```
         Cheks and transforms array of targets. Targets are transformed in place.
         Args:
           class_names(list):  labels associated with targets (e.g., ['negative', 'positive'])
@@ -546,6 +556,7 @@ class YTransform:
                          2. targets are strings and task is classification (class_names are populated automatically)
           label_encoder(LabelEncoder): a prior instance of LabelEncoder.  
                                        If None, will be created when train=True
+        ```
         """
         if type(class_names) != list:
             if isinstance(class_names, (pd.Series, np.ndarray)): class_names = class_names.tolist()
@@ -604,9 +615,8 @@ class YTransform:
             if np.issubdtype(type(max(targets)), np.floating):
                 warnings.warn('class_names implies classification but targets array contains float(s) instead of integers or strings')
 
-            # TODO check logic
-            if train and ( len(set(targets)) != len(list(range(int(max(targets)+1)))) ):
-                raise ValueError('len(set(targets) is %s but len(list(range(int(max(targets)+1))) is  %s' % (len(list(set(targets))), len(list(range(int(max(targets))+1)))))
+            if train and ( len(set(targets)) != int(max(targets)+1) ):
+                raise ValueError('len(set(targets) is %s but max(targets)+1 is  %s' % ( len(set(targets)), int(max(targets)+1) ))
             targets = to_categorical(targets, num_classes=len(self.get_classes()))
         if train: self.train_called=True
         return targets
@@ -622,11 +632,13 @@ class YTransform:
 class YTransformDataFrame(YTransform):
     def __init__(self, label_columns=[], is_regression=False):
         """
+        ```
         Checks and transforms label columns in DataFrame. DataFrame is modified in place
         Args:
           label_columns(list): list of columns storing labels 
           is_regression(bool): If True, task is regression and integer targets are treated as numeric dependent variable.
                                IF False, task is classification and integer targets are treated as class IDs.
+        ```
         """
         self.is_regression = is_regression
         if isinstance(label_columns, str): label_columns = [label_columns]
@@ -705,6 +717,8 @@ class YTransformDataFrame(YTransform):
         for i, col in enumerate(new_lab_cols):
             df[col] = targets[:,i]
         df[new_lab_cols] = targets
+        print(new_lab_cols)
+        print(df[new_lab_cols].head())
         df[new_lab_cols] = df[new_lab_cols].astype('float32')
 
         return df
